@@ -1,7 +1,7 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab:
 /*
 Gwenview: an image viewer
-Copyright 2008 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2008 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,7 +35,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 namespace Gwenview {
 
 
-class KIPIImageCollectionSelectorPrivate;
+struct KIPIImageCollectionSelectorPrivate;
 class KIPIImageCollectionSelector : public KIPI::ImageCollectionSelector {
 	Q_OBJECT
 public:
