@@ -22,6 +22,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include "thumbnailviewpanel.moc"
 
 // Qt
+#include <QDropEvent>
 #include <QMenu>
 #include <QSlider>
 #include <QToolTip>
@@ -43,6 +44,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 
 // Local
 #include <filtercontroller.h>
+#include <fileoperations.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/semanticinfo/abstractsemanticinfobackend.h>
 #include <lib/semanticinfo/sorteddirmodel.h>
@@ -91,12 +93,12 @@ struct ThumbnailViewPanelPrivate : public Ui_ThumbnailViewPanel {
 		QVBoxLayout* layout = new QVBoxLayout(mUrlNavigatorContainer);
 		layout->setMargin(0);
 		layout->addWidget(mUrlNavigator);
+		QObject::connect(mUrlNavigator, SIGNAL(urlsDropped(const KUrl&, QDropEvent*)),
+			that, SLOT(slotUrlsDropped(const KUrl&, QDropEvent*)));
 
 		// Thumbnail slider
 		QObject::connect(mThumbnailSlider, SIGNAL(valueChanged(int)),
-			mThumbnailView, SLOT(setThumbnailSize(int)) );
-		QObject::connect(mThumbnailSlider, SIGNAL(actionTriggered(int)),
-			that, SLOT(updateSliderToolTip(int)) );
+			mThumbnailView, SLOT(setThumbnailSize(int)));
 	}
 
 	void setupActions(KActionCollection* actionCollection) {
@@ -209,7 +211,7 @@ void ThumbnailViewPanel::loadConfig() {
 	d->mUrlNavigator->setShowFullPath(GwenviewConfig::urlNavigatorShowFullPath());
 
 	d->mThumbnailSlider->setValue(GwenviewConfig::thumbnailSize());
-	updateSliderToolTip(QAbstractSlider::SliderNoAction);
+	d->mThumbnailSlider->updateToolTip();
 	// If GwenviewConfig::thumbnailSize() returns the current value of
 	// mThumbnailSlider, it won't emit valueChanged() and the thumbnail view
 	// won't be updated. That's why we do it ourself.
@@ -311,18 +313,22 @@ void ThumbnailViewPanel::applyPalette(const QPalette& palette) {
 }
 
 
-void ThumbnailViewPanel::updateSliderToolTip(int actionTriggered) {
-	// FIXME: i18n?
-	const int size = d->mThumbnailSlider->sliderPosition();
-	const QString text = QString("%1 x %2").arg(size).arg(size);
-	d->mThumbnailSlider->setToolTip(text);
-
-	if (actionTriggered != QAbstractSlider::SliderNoAction) {
-		// If we are updating because of a direct action on the slider, show
-		// the tooltip immediatly.
-		const QPoint pos = d->mThumbnailSlider->mapToGlobal(QPoint(0, d->mThumbnailSlider->height() / 2));
-		QToolTip::showText(pos, text, d->mThumbnailSlider);
+void ThumbnailViewPanel::slotUrlsDropped(const KUrl& destUrl, QDropEvent* event) {
+	const KUrl::List urlList = KUrl::List::fromMimeData(event->mimeData());
+	if (urlList.isEmpty()) {
+		return;
 	}
+	event->acceptProposedAction();
+
+	// We can't call FileOperations::showMenuForDroppedUrls() directly because
+	// we need the slot to return so that the drop event is accepted. Otherwise
+	// the drop cursor is still visible when the menu is shown.
+	QMetaObject::invokeMethod(this, "showMenuForDroppedUrls", Qt::QueuedConnection, Q_ARG(KUrl::List, urlList), Q_ARG(KUrl, destUrl));
+}
+
+
+void ThumbnailViewPanel::showMenuForDroppedUrls(const KUrl::List& urlList, const KUrl& destUrl) {
+	FileOperations::showMenuForDroppedUrls(d->mUrlNavigator, urlList, destUrl);
 }
 
 
