@@ -61,7 +61,7 @@ bool KExiv2::hasXmp() const
 {
 #ifdef _XMP_SUPPORT_
 
-    return !d->xmpMetadata.empty();
+    return !d->xmpMetadata().empty();
 
 #else
 
@@ -76,7 +76,7 @@ bool KExiv2::clearXmp() const
 
     try
     {
-        d->xmpMetadata.clear();
+        d->xmpMetadata().clear();
         return true;
     }
     catch( Exiv2::Error &e )
@@ -95,11 +95,11 @@ QByteArray KExiv2::getXmp() const
 
     try
     {
-        if (!d->xmpMetadata.empty())
+        if (!d->xmpMetadata().empty())
         {
 
             std::string xmpPacket;
-            Exiv2::XmpParser::encode(xmpPacket, d->xmpMetadata);
+            Exiv2::XmpParser::encode(xmpPacket, d->xmpMetadata());
             QByteArray data(xmpPacket.data(), xmpPacket.size());
             return data;
         }
@@ -128,7 +128,7 @@ bool KExiv2::setXmp(const QByteArray& data) const
 
             std::string xmpPacket;
             xmpPacket.assign(data.data(), data.size());
-            if (Exiv2::XmpParser::decode(d->xmpMetadata, xmpPacket) != 0)
+            if (Exiv2::XmpParser::decode(d->xmpMetadata(), xmpPacket) != 0)
                 return false;
             else
                 return true;
@@ -151,12 +151,12 @@ KExiv2::MetaDataMap KExiv2::getXmpTagsDataList(const QStringList &xmpKeysFilter,
 {
 #ifdef _XMP_SUPPORT_
 
-    if (d->xmpMetadata.empty())
+    if (d->xmpMetadata().empty())
        return MetaDataMap();
 
     try
     {
-        Exiv2::XmpData xmpData = d->xmpMetadata;
+        Exiv2::XmpData xmpData = d->xmpMetadata();
         xmpData.sortByKey();
 
         QString     ifDItemName;
@@ -278,7 +278,7 @@ QString KExiv2::getXmpTagString(const char* xmpTagName, bool escapeCR) const
 
     try
     {
-        Exiv2::XmpData xmpData(d->xmpMetadata);
+        Exiv2::XmpData xmpData(d->xmpMetadata());
         Exiv2::XmpKey key(xmpTagName);
         Exiv2::XmpData::iterator it = xmpData.findKey(key);
         if (it != xmpData.end())
@@ -317,7 +317,7 @@ bool KExiv2::setXmpTagString(const char *xmpTagName, const QString& value, bool
         Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::xmpText);
         xmpTxtVal->read(txt);
 
-        d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
         return true;
     }
     catch( Exiv2::Error &e )
@@ -336,7 +336,7 @@ KExiv2::AltLangMap KExiv2::getXmpTagStringListLangAlt(const char* xmpTagName, bo
 
     try
     {
-        Exiv2::XmpData xmpData = d->xmpMetadata;
+        Exiv2::XmpData xmpData = d->xmpMetadata();
 
         for (Exiv2::XmpData::iterator it = xmpData.begin(); it != xmpData.end(); ++it)
         {
@@ -388,7 +388,7 @@ bool KExiv2::setXmpTagStringListLangAlt(const char *xmpTagName, const KExiv2::Al
         {
             Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::langAlt);
 
-            for (AltLangMap::const_iterator it = values.begin(); it != values.end(); ++it)
+            for (AltLangMap::const_iterator it = values.constBegin(); it != values.constEnd(); ++it)
             {
                 QString lang = it.key();
                 QString text = it.value();
@@ -398,7 +398,7 @@ bool KExiv2::setXmpTagStringListLangAlt(const char *xmpTagName, const KExiv2::Al
             }
 
             // ...and add the new one instead.
-            d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+            d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
         }
         return true;
     }
@@ -418,7 +418,7 @@ QString KExiv2::getXmpTagStringLangAlt(const char* xmpTagName, const QString& la
 
     try
     {
-        Exiv2::XmpData xmpData(d->xmpMetadata);
+        Exiv2::XmpData xmpData(d->xmpMetadata());
         Exiv2::XmpKey key(xmpTagName);
         for (Exiv2::XmpData::iterator it = xmpData.begin(); it != xmpData.end(); ++it)
         {
@@ -491,7 +491,7 @@ bool KExiv2::setXmpTagStringLangAlt(const char *xmpTagName, const QString& value
 
         xmpTxtVal->read(txt);
         removeXmpTag(xmpTagName);
-        d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
         return true;
     }
     catch( Exiv2::Error &e )
@@ -510,7 +510,7 @@ QStringList KExiv2::getXmpTagStringSeq(const char* xmpTagName, bool escapeCR) co
 
     try
     {
-        Exiv2::XmpData xmpData(d->xmpMetadata);
+        Exiv2::XmpData xmpData(d->xmpMetadata());
         Exiv2::XmpKey key(xmpTagName);
         Exiv2::XmpData::iterator it = xmpData.findKey(key);
         if (it != xmpData.end())
@@ -571,7 +571,7 @@ bool KExiv2::setXmpTagStringSeq(const char *xmpTagName, const QStringList& seq,
                 const std::string &txt((*it).toUtf8().constData());
                 xmpTxtSeq->read(txt);
             }
-            d->xmpMetadata[xmpTagName].setValue(xmpTxtSeq.get());
+            d->xmpMetadata()[xmpTagName].setValue(xmpTxtSeq.get());
             return true;
         }
     }
@@ -591,7 +591,7 @@ QStringList KExiv2::getXmpTagStringBag(const char* xmpTagName, bool escapeCR) co
 
     try
     {
-        Exiv2::XmpData xmpData(d->xmpMetadata);
+        Exiv2::XmpData xmpData(d->xmpMetadata());
         Exiv2::XmpKey key(xmpTagName);
         Exiv2::XmpData::iterator it = xmpData.findKey(key);
         if (it != xmpData.end())
@@ -651,7 +651,7 @@ bool KExiv2::setXmpTagStringBag(const char *xmpTagName, const QStringList& bag,
                 const std::string &txt((*it).toUtf8().constData());
                 xmpTxtBag->read(txt);
             }
-            d->xmpMetadata[xmpTagName].setValue(xmpTxtBag.get());
+            d->xmpMetadata()[xmpTagName].setValue(xmpTxtBag.get());
             return true;
         }
     }
@@ -665,12 +665,64 @@ bool KExiv2::setXmpTagStringBag(const char *xmpTagName, const QStringList& bag,
     return false;
 }
 
+bool KExiv2::addToXmpTagStringBag(const char *xmpTagName, const QStringList& entriesToAdd,
+                                     bool setProgramName) const
+{
+//#ifdef _XMP_SUPPORT_
+
+    if (!setProgramId(setProgramName))
+        return false;
+
+    QStringList oldEntries = getXmpTagStringBag(xmpTagName, false);
+    QStringList newEntries = entriesToAdd;
+
+    // Create a list of keywords including old one which already exists.
+    for (QStringList::const_iterator it = oldEntries.constBegin(); it != oldEntries.constEnd(); ++it )
+    {
+        if (!newEntries.contains(*it))
+            newEntries.append(*it);
+    }
+
+    if (setXmpTagStringBag(xmpTagName, newEntries, false))
+        return true;
+
+//#endif // _XMP_SUPPORT_
+
+    return false;
+}
+
+bool KExiv2::removeFromXmpTagStringBag(const char *xmpTagName, const QStringList& entriesToRemove,
+                                          bool setProgramName) const
+{
+//#ifdef _XMP_SUPPORT_
+
+    if (!setProgramId(setProgramName))
+        return false;
+
+    QStringList currentEntries = getXmpTagStringBag(xmpTagName, false);
+    QStringList newEntries;
+
+    // Create a list of current keywords except those that shall be removed
+    for (QStringList::const_iterator it = currentEntries.constBegin(); it != currentEntries.constEnd(); ++it )
+    {
+        if (!entriesToRemove.contains(*it))
+            newEntries.append(*it);
+    }
+
+    if (setXmpTagStringBag(xmpTagName, newEntries, false))
+        return true;
+
+//#endif // _XMP_SUPPORT_
+
+    return false;
+}
+
 QVariant KExiv2::getXmpTagVariant(const char *xmpTagName, bool rationalAsListOfInts, bool stringEscapeCR) const
 {
 #ifdef _XMP_SUPPORT_
     try
     {
-        Exiv2::XmpData xmpData(d->xmpMetadata);
+        Exiv2::XmpData xmpData(d->xmpMetadata());
         Exiv2::XmpKey key(xmpTagName);
         Exiv2::XmpData::iterator it = xmpData.findKey(key);
         if (it != xmpData.end())
@@ -771,7 +823,7 @@ QVariant KExiv2::getXmpTagVariant(const char *xmpTagName, bool rationalAsListOfI
     return QVariant();
 }
 
-bool KExiv2::registerXmpNameSpace(const QString& uri, const QString& prefix) const
+bool KExiv2::registerXmpNameSpace(const QString& uri, const QString& prefix)
 {
 #ifdef _XMP_SUPPORT_
 
@@ -784,7 +836,28 @@ bool KExiv2::registerXmpNameSpace(const QString& uri, const QString& prefix) con
     }
     catch( Exiv2::Error &e )
     {
-        d->printExiv2ExceptionError("Cannot register a new Xmp namespace using Exiv2 ", e);
+        KExiv2Priv::printExiv2ExceptionError("Cannot register a new Xmp namespace using Exiv2 ", e);
+    }
+
+#endif // _XMP_SUPPORT_
+
+    return false;
+}
+
+bool KExiv2::unregisterXmpNameSpace(const QString& uri)
+{
+#ifdef _XMP_SUPPORT_
+
+    try
+    {
+        QString ns = uri;
+        if (!uri.endsWith("/")) ns.append("/");
+        Exiv2::XmpProperties::unregisterNs(ns.toAscii().constData());
+        return true;
+    }
+    catch( Exiv2::Error &e )
+    {
+        KExiv2Priv::printExiv2ExceptionError("Cannot unregister a new Xmp namespace using Exiv2 ", e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -802,10 +875,10 @@ bool KExiv2::removeXmpTag(const char *xmpTagName, bool setProgramName) const
     try
     {
         Exiv2::XmpKey xmpKey(xmpTagName);
-        Exiv2::XmpData::iterator it = d->xmpMetadata.findKey(xmpKey);
-        if (it != d->xmpMetadata.end())
+        Exiv2::XmpData::iterator it = d->xmpMetadata().findKey(xmpKey);
+        if (it != d->xmpMetadata().end())
         {
-            d->xmpMetadata.erase(it);
+            d->xmpMetadata().erase(it);
             return true;
         }
     }
@@ -826,27 +899,12 @@ QStringList KExiv2::getXmpKeywords() const
 
 bool KExiv2::setXmpKeywords(const QStringList& newKeywords, bool setProgramName) const
 {
-#ifdef _XMP_SUPPORT_
+    return addToXmpTagStringBag("Xmp.dc.subject", newKeywords, setProgramName);
+}
 
-    if (!setProgramId(setProgramName))
-        return false;
-
-    QStringList oldkeys = getXmpKeywords();
-    QStringList newkeys = newKeywords;
-
-    // Create a list of keywords including old one which already exists.
-    for (QStringList::const_iterator it = oldkeys.constBegin(); it != oldkeys.constEnd(); ++it )
-    {
-        if (!newkeys.contains(*it))
-            newkeys.append(*it);
-    }
-
-    if (setXmpTagStringBag("Xmp.dc.subject", newkeys, false))
-        return true;
-
-#endif // _XMP_SUPPORT_
-
-    return false;
+bool KExiv2::removeXmpKeywords(const QStringList& keywordsToRemove, bool setProgramName)
+{
+    return removeFromXmpTagStringBag("Xmp.dc.subject", keywordsToRemove, setProgramName);
 }
 
 QStringList KExiv2::getXmpSubCategories() const
@@ -856,27 +914,12 @@ QStringList KExiv2::getXmpSubCategories() const
 
 bool KExiv2::setXmpSubCategories(const QStringList& newSubCategories, bool setProgramName) const
 {
-#ifdef _XMP_SUPPORT_
+    return addToXmpTagStringBag("Xmp.photoshop.SupplementalCategories", newSubCategories, setProgramName);
+}
 
-    if (!setProgramId(setProgramName))
-        return false;
-
-    const QStringList oldSubCat = getXmpSubCategories();
-    QStringList newSubCat = newSubCategories;
-
-    // Create a list of sub-categories including old one which already exists.
-    for (QStringList::const_iterator it = oldSubCat.constBegin(); it != oldSubCat.constEnd(); ++it )
-    {
-        if (!newSubCat.contains(*it))
-            newSubCat.append(*it);
-    }
-
-    if (setXmpTagStringBag("Xmp.photoshop.SupplementalCategories", newSubCat, false))
-        return true;
-
-#endif // _XMP_SUPPORT_
-
-    return false;
+bool KExiv2::removeXmpSubCategories(const QStringList& subCategoriesToRemove, bool setProgramName)
+{
+    return removeFromXmpTagStringBag("Xmp.photoshop.SupplementalCategories", subCategoriesToRemove, setProgramName);
 }
 
 QStringList KExiv2::getXmpSubjects() const
@@ -886,27 +929,48 @@ QStringList KExiv2::getXmpSubjects() const
 
 bool KExiv2::setXmpSubjects(const QStringList& newSubjects, bool setProgramName) const
 {
-#ifdef _XMP_SUPPORT_
+    return addToXmpTagStringBag("Xmp.iptc.SubjectCode", newSubjects, setProgramName);
+}
 
-    if (!setProgramId(setProgramName))
-        return false;
+bool KExiv2::removeXmpSubjects(const QStringList& subjectsToRemove, bool setProgramName)
+{
+    return removeFromXmpTagStringBag("Xmp.iptc.SubjectCode", subjectsToRemove, setProgramName);
+}
 
-    QStringList oldSubjectCodes = getXmpSubjects();
-    QStringList newSubjectCodes = newSubjects;
-
-    // Create a list of sub-categories including old one which already exists.
-    for (QStringList::const_iterator it = oldSubjectCodes.constBegin(); it != oldSubjectCodes.constEnd(); ++it )
+KExiv2::TagsMap KExiv2::getXmpTagsList() const
+{
+    try
     {
-        if (!newSubjectCodes.contains(*it))
-            newSubjectCodes.append(*it);
+        TagsMap tagsMap;
+        d->getXMPTagsListFromPrefix("dc",             tagsMap);
+        d->getXMPTagsListFromPrefix("digiKam",        tagsMap);
+        d->getXMPTagsListFromPrefix("xmp",            tagsMap);
+        d->getXMPTagsListFromPrefix("xmpRights",      tagsMap);
+        d->getXMPTagsListFromPrefix("xmpMM",          tagsMap);
+        d->getXMPTagsListFromPrefix("xmpBJ",          tagsMap);
+        d->getXMPTagsListFromPrefix("xmpTPg",         tagsMap);
+        d->getXMPTagsListFromPrefix("xmpDM",          tagsMap);
+        d->getXMPTagsListFromPrefix("MicrosoftPhoto", tagsMap);
+        d->getXMPTagsListFromPrefix("pdf",            tagsMap);
+        d->getXMPTagsListFromPrefix("photoshop",      tagsMap);
+        d->getXMPTagsListFromPrefix("crs",            tagsMap);
+        d->getXMPTagsListFromPrefix("tiff",           tagsMap);
+        d->getXMPTagsListFromPrefix("exif",           tagsMap);
+        d->getXMPTagsListFromPrefix("aux",            tagsMap);
+        d->getXMPTagsListFromPrefix("iptc",           tagsMap);
+
+#if (EXIV2_TEST_VERSION(0,19,0))
+        d->getXMPTagsListFromPrefix("iptcExt",        tagsMap);
+        d->getXMPTagsListFromPrefix("plus",           tagsMap);
+#endif
+        return tagsMap;
+    }
+    catch( Exiv2::Error &e )
+    {
+        d->printExiv2ExceptionError("Cannot get Xmp Tags list using Exiv2 ", e);
     }
 
-    if (setXmpTagStringBag("Xmp.iptc.SubjectCode", newSubjectCodes, false))
-        return true;
-
-#endif // _XMP_SUPPORT_
-
-    return false;
+    return TagsMap();
 }
 
 }  // NameSpace KExiv2Iface
