@@ -116,13 +116,13 @@ bool KSaneOption::writeData(void *data)
 
     status = sane_control_option (m_handle, m_index, SANE_ACTION_SET_VALUE, data, &res);
     if (status != SANE_STATUS_GOOD) {
-        kDebug(51004) << m_optDesc->name << "sane_control_option returned:" << sane_strstatus(status);
+        kDebug() << m_optDesc->name << "sane_control_option returned:" << sane_strstatus(status);
         // write failed. re read the current setting
         readValue();
         return false;
     }
     if ((res & SANE_INFO_INEXACT) && (m_frame != 0)) {
-        //kDebug(51004) << "write was inexact. Reload value just in case...";
+        //kDebug() << "write was inexact. Reload value just in case...";
         readValue();
     }
 
@@ -165,6 +165,7 @@ bool KSaneOption::getValue(float &) {return false;}
 bool KSaneOption::setValue(float) {return false;}
 bool KSaneOption::getValue(QString &) {return false;}
 bool KSaneOption::setValue(const QString &) {return false;}
+int  KSaneOption::getUnit() {return m_optDesc->unit;}
 
 bool KSaneOption::storeCurrentData()
 {
@@ -180,7 +181,7 @@ bool KSaneOption::storeCurrentData()
     m_data = (unsigned char *)malloc(m_optDesc->size);
     status = sane_control_option (m_handle, m_index, SANE_ACTION_GET_VALUE, m_data, &res);
     if (status != SANE_STATUS_GOOD) {
-        kDebug(51004) << m_optDesc->name << "sane_control_option returned" << status;
+        kDebug() << m_optDesc->name << "sane_control_option returned" << status;
         return false;
     }
     return true;
@@ -203,12 +204,12 @@ bool KSaneOption::restoreSavedData()
 
 void KSaneOption::widgetSizeHints(int *, int *)
 {
-    kDebug(51004) << m_optDesc->name << " : type not supported";
+    kDebug() << m_optDesc->name << " : type not supported";
 }
 
 void KSaneOption::setColumnWidths(int, int)
 {
-    kDebug(51004) << m_optDesc->name << " : type not supported";
+    kDebug() << m_optDesc->name << " : type not supported";
 }
 
 KSaneOption::KSaneOptType KSaneOption::otpionType(const SANE_Option_Descriptor *optDesc)
@@ -223,15 +224,15 @@ KSaneOption::KSaneOptType KSaneOption::otpionType(const SANE_Option_Descriptor *
                     return TYPE_CHECKBOX;
                 case SANE_TYPE_INT:
                     if (optDesc->size == sizeof(SANE_Word)) return TYPE_SLIDER;
-                    kDebug(51004) << "Can not handle:"<< optDesc->title;
-                    kDebug(51004) << "SANE_CONSTRAINT_NONE && SANE_TYPE_INT";
-                    kDebug(51004) << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
+                    kDebug() << "Can not handle:"<< optDesc->title;
+                    kDebug() << "SANE_CONSTRAINT_NONE && SANE_TYPE_INT";
+                    kDebug() << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
                     break;
                 case SANE_TYPE_FIXED:
                     if (optDesc->size == sizeof(SANE_Word)) return TYPE_F_SLIDER;
-                    kDebug(51004) << "Can not handle:"<< optDesc->title;
-                    kDebug(51004) << "SANE_CONSTRAINT_NONE && SANE_TYPE_FIXED";
-                    kDebug(51004) << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
+                    kDebug() << "Can not handle:"<< optDesc->title;
+                    kDebug() << "SANE_CONSTRAINT_NONE && SANE_TYPE_FIXED";
+                    kDebug() << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
                     break;
                 case SANE_TYPE_BUTTON:
                     return TYPE_BUTTON;
@@ -255,20 +256,20 @@ KSaneOption::KSaneOptType KSaneOption::otpionType(const SANE_Option_Descriptor *
                     {
                         return TYPE_GAMMA;
                     }
-                    kDebug(51004) << "Can not handle:"<< optDesc->title;
-                    kDebug(51004) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_INT && !SANE_NAME_GAMMA_VECTOR...";
-                    kDebug(51004) << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
+                    kDebug() << "Can not handle:"<< optDesc->title;
+                    kDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_INT && !SANE_NAME_GAMMA_VECTOR...";
+                    kDebug() << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
                     break;
                 case SANE_TYPE_FIXED:
                     if (optDesc->size == sizeof(SANE_Word)) return TYPE_F_SLIDER;
-                    kDebug(51004) << "Can not handle:"<< optDesc->title;
-                    kDebug(51004) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_FIXED";
-                    kDebug(51004) << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
-                    kDebug(51004) << "Analog Gamma vector?";
+                    kDebug() << "Can not handle:"<< optDesc->title;
+                    kDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_FIXED";
+                    kDebug() << "size" << optDesc->size<< "!= sizeof(SANE_Word)";
+                    kDebug() << "Analog Gamma vector?";
                     break;
                 case SANE_TYPE_STRING:
-                    kDebug(51004) << "Can not handle:" << optDesc->title;
-                    kDebug(51004) << "SANE_CONSTRAINT_RANGE && SANE_TYPE_STRING";
+                    kDebug() << "Can not handle:" << optDesc->title;
+                    kDebug() << "SANE_CONSTRAINT_RANGE && SANE_TYPE_STRING";
                     return TYPE_DETECT_FAIL;
                 case SANE_TYPE_BUTTON:
                     return TYPE_BUTTON;
@@ -283,17 +284,32 @@ KSaneOption::KSaneOptType KSaneOption::otpionType(const SANE_Option_Descriptor *
     return TYPE_DETECT_FAIL;
 }
 
-QString KSaneOption::unitString()
+KLocalizedString KSaneOption::unitString()
+{
+    switch(m_optDesc->unit)
+    {
+        case SANE_UNIT_NONE:        return KLocalizedString();
+        case SANE_UNIT_PIXEL:       return ki18ncp("SpinBox parameter unit", " Pixel", " Pixels");
+        case SANE_UNIT_BIT:         return ki18ncp("SpinBox parameter unit", " Bit", " Bits");
+        case SANE_UNIT_MM:          return ki18nc("SpinBox parameter unit", " mm");
+        case SANE_UNIT_DPI:         return ki18nc("SpinBox parameter unit", " DPI");
+        case SANE_UNIT_PERCENT:     return ki18nc("SpinBox parameter unit", " %");
+        case SANE_UNIT_MICROSECOND: return ki18ncp("SpinBox parameter unit", " usec", " usecs");
+    }
+    return KLocalizedString();
+}
+
+QString KSaneOption::unitDoubleString()
 {
     switch(m_optDesc->unit)
     {
         case SANE_UNIT_NONE:        return QString("");
-        case SANE_UNIT_PIXEL:       return i18nc("SpinBox parameter unit"," Pixels");
-        case SANE_UNIT_BIT:         return i18nc("SpinBox parameter unit"," Bits");
-        case SANE_UNIT_MM:          return i18nc("SpinBox parameter unit"," mm");
-        case SANE_UNIT_DPI:         return i18nc("SpinBox parameter unit"," DPI");
-        case SANE_UNIT_PERCENT:     return i18nc("SpinBox parameter unit"," %");
-        case SANE_UNIT_MICROSECOND: return i18nc("SpinBox parameter unit"," usec");
+        case SANE_UNIT_PIXEL:       return i18nc("Double numbers. SpinBox parameter unit", " Pixels");
+        case SANE_UNIT_BIT:         return i18nc("Double numbers. SpinBox parameter unit", " Bits");
+        case SANE_UNIT_MM:          return i18nc("Double numbers. SpinBox parameter unit", " mm");
+        case SANE_UNIT_DPI:         return i18nc("Double numbers. SpinBox parameter unit", " DPI");
+        case SANE_UNIT_PERCENT:     return i18nc("Double numbers. SpinBox parameter unit", " %");
+        case SANE_UNIT_MICROSECOND: return i18nc("Double numbers. SpinBox parameter unit", " usec");
     }
     return QString("");
 }
