@@ -36,6 +36,24 @@ class SearchPoint
         int offset_end;
 };
 
+/* text comparison functions */
+
+bool CaseInsensitiveCmpFn( const QStringRef & from, const QStringRef & to,
+                           int *fromLength, int *toLength )
+{
+    *fromLength = from.length();
+    *toLength = to.length();
+    return from.compare( to, Qt::CaseInsensitive ) == 0;
+}
+
+bool CaseSensitiveCmpFn( const QStringRef & from, const QStringRef & to,
+                           int *fromLength, int *toLength )
+{
+    *fromLength = from.length();
+    *toLength = to.length();
+    return from.compare( to, Qt::CaseSensitive ) == 0;
+}
+
 
 /*
   Rationale behind TinyTextEntity:
@@ -44,27 +62,56 @@ class SearchPoint
   we store the UTF-16 data and their length. This way, we save about
   4 int's wrt a QString, and we can create a new string from that
   raw data (that's the only penalty of that).
+  Even better, if the string we need to store has at most
+  MaxStaticChars characters, then we store those in place of the QChar*
+  that would be used (with new[] + free[]) for the data.
  */
 class TinyTextEntity
 {
+    static const int MaxStaticChars = sizeof( QChar * ) / sizeof( QChar );
+
     public:
         TinyTextEntity( const QString &text, const NormalizedRect &rect )
             : area( rect )
         {
             Q_ASSERT_X( !text.isEmpty(), "TinyTextEntity", "empty string" );
+            Q_ASSERT_X( sizeof( d ) == sizeof( QChar * ), "TinyTextEntity",
+                        "internal storage is wider than QChar*, fix it!" );
             length = text.length();
-            data = new QChar[ length ];
-            std::memcpy( data, text.constData(), length * sizeof( QChar ) );
+            switch ( length )
+            {
+#if QT_POINTER_SIZE >= 8
+                case 4:
+                    d.qc[3] = text.at( 3 ).unicode();
+                    // fall through
+                case 3:
+                    d.qc[2] = text.at( 2 ).unicode();
+                    // fall through
+#endif
+                case 2:
+                    d.qc[1] = text.at( 1 ).unicode();
+                    // fall through
+                case 1:
+                    d.qc[0] = text.at( 0 ).unicode();
+                    break;
+                default:
+                    d.data = new QChar[ length ];
+                    std::memcpy( d.data, text.constData(), length * sizeof( QChar ) );
+            }
         }
 
         ~TinyTextEntity()
         {
-            delete [] data;
+            if ( length > MaxStaticChars )
+            {
+                delete [] d.data;
+            }
         }
 
         inline QString text() const
         {
-            return QString::fromRawData( data, length );
+            return length <= MaxStaticChars ? QString::fromRawData( ( const QChar * )&d.qc[0], length )
+                                            : QString::fromRawData( d.data, length );
         }
 
         inline NormalizedRect transformedArea( const QMatrix &matrix ) const
@@ -79,7 +126,11 @@ class TinyTextEntity
     private:
         Q_DISABLE_COPY( TinyTextEntity )
 
-        QChar *data;
+        union
+        {
+            QChar *data;
+            ushort qc[MaxStaticChars];
+        } d;
         int length;
 };
 
@@ -132,7 +183,7 @@ TextPage::TextPage()
 TextPage::TextPage( const TextEntity::List &words )
     : d( new TextPagePrivate() )
 {
-    TextEntity::List::ConstIterator it = words.begin(), itEnd = words.end();
+    TextEntity::List::ConstIterator it = words.constBegin(), itEnd = words.constEnd();
     for ( ; it != itEnd; ++it )
     {
         TextEntity *e = *it;
@@ -181,7 +232,7 @@ RegularAreaRect * TextPage::textArea ( TextSelection * sel) const
     int itB = -1;
     int itE = -1;
 
-    // ending cursor is higher then start cursor, we need to find positions in reverse
+    // ending cursor is higher than start cursor, we need to find positions in reverse
     NormalizedRect tmp;
     NormalizedRect start;
     NormalizedRect end;
@@ -257,7 +308,7 @@ RegularAreaRect * TextPage::textArea ( TextSelection * sel) const
         end = *d->m_words[ sel->itE() ]->area();
 
         NormalizedRect first, second, third;
-        /// finding out if there are more then one baseline between them is a hard and discussable task
+        /// finding out if there is more than one baseline between them is a hard and discussable task
         /// we will create a rectangle (rx,0)x(tx,1) and will check how many times does it intersect the 
         /// areas, if more than one -> we have a three or over line selection
         first = start;
@@ -285,7 +336,7 @@ RegularAreaRect * TextPage::textArea ( TextSelection * sel) const
     double endCx = endC.x;
     double endCy = endC.y;
 
-    TextList::ConstIterator it = d->m_words.begin(), itEnd = d->m_words.end();
+    TextList::ConstIterator it = d->m_words.constBegin(), itEnd = d->m_words.constEnd();
     MergeSide side = d->m_page ? (MergeSide)d->m_page->m_page->totalOrientation() : MergeRight;
     for ( ; it != itEnd; ++it )
     {
@@ -311,8 +362,8 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
         return 0;
     TextList::ConstIterator start;
     TextList::ConstIterator end;
-    QMap< int, SearchPoint* >::const_iterator sIt = d->m_searchPoints.find( searchID );
-    if ( sIt == d->m_searchPoints.end() )
+    QMap< int, SearchPoint* >::const_iterator sIt = d->m_searchPoints.constFind( searchID );
+    if ( sIt == d->m_searchPoints.constEnd() )
     {
         // if no previous run of this search is found, then set it to start
         // from the beginning (respecting the search direction)
@@ -325,12 +376,12 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
     switch ( dir )
     {
         case FromTop:
-            start = d->m_words.begin();
-            end = d->m_words.end();
+            start = d->m_words.constBegin();
+            end = d->m_words.constEnd();
             break;
         case FromBottom:
-            start = d->m_words.end();
-            end = d->m_words.begin();
+            start = d->m_words.constEnd();
+            end = d->m_words.constBegin();
             Q_ASSERT( start != end );
             // we can safely go one step back, as we already checked
             // that the list is not empty
@@ -339,26 +390,28 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
             break;
         case NextResult:
             start = (*sIt)->it_end;
-            end = d->m_words.end();
+            end = d->m_words.constEnd();
             if ( ( start + 1 ) != end )
                 ++start;
             break;
         case PreviousResult:
             start = (*sIt)->it_begin;
-            end = d->m_words.begin();
+            end = d->m_words.constBegin();
             if ( start != end )
                 --start;
             forward = false;
             break;
     };
     RegularAreaRect* ret = 0;
+    const TextComparisonFunction cmpFn = caseSensitivity == Qt::CaseSensitive
+                                       ? CaseSensitiveCmpFn : CaseInsensitiveCmpFn;
     if ( forward )
     {
-        ret = d->findTextInternalForward( searchID, query, caseSensitivity, start, end );
+        ret = d->findTextInternalForward( searchID, query, caseSensitivity, cmpFn, start, end );
     }
     else
     {
-        ret = d->findTextInternalBackward( searchID, query, caseSensitivity, start, end );
+        ret = d->findTextInternalBackward( searchID, query, caseSensitivity, cmpFn, start, end );
     }
     return ret;
 }
@@ -366,6 +419,7 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
 
 RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const QString &_query,
                                                              Qt::CaseSensitivity caseSensitivity,
+                                                             TextComparisonFunction comparer,
                                                              const TextList::ConstIterator &start,
                                                              const TextList::ConstIterator &end )
 {
@@ -402,13 +456,12 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
 #ifdef DEBUG_TEXTPAGE
             kDebug(OkularDebug) << str.mid(offset,min) << ":" << _query.mid(j,min);
 #endif
-            // we have equal (or less then) area of the query left as the lengt of the current 
+            // we have equal (or less than) area of the query left as the length of the current 
             // entity
 
-            if ((caseSensitivity == Qt::CaseSensitive)
-                ? (str.mid(offset,min) != query.mid(j,min))
-                : (str.mid(offset,min).toLower() != query.mid(j,min))
-                )
+            int resStrLen = 0, resQueryLen = 0;
+            if ( !comparer( str.midRef( offset, min ), query.midRef( j, min ),
+                            &resStrLen, &resQueryLen ) )
             {
                     // we not have matched
                     // this means we do not have a complete match
@@ -437,8 +490,8 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
 #endif
                     haveMatch=true;
                     ret->append( curEntity->transformedArea( matrix ) );
-                    j+=min;
-                    queryLeft-=min;
+                    j += resStrLen;
+                    queryLeft -= resQueryLen;
                     if ( it_begin == TextList::ConstIterator() )
                     {
                         it_begin = it;
@@ -476,6 +529,7 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
 
 RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const QString &_query,
                                                             Qt::CaseSensitivity caseSensitivity,
+                                                            TextComparisonFunction comparer,
                                                             const TextList::ConstIterator &start,
                                                             const TextList::ConstIterator &end )
 {
@@ -528,13 +582,12 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
 #ifdef DEBUG_TEXTPAGE
             kDebug(OkularDebug) << str.right(min) << " : " << _query.mid(j-min+1,min);
 #endif
-            // we have equal (or less then) area of the query left as the lengt of the current 
+            // we have equal (or less than) area of the query left as the length of the current 
             // entity
 
-            if ((caseSensitivity == Qt::CaseSensitive)
-                ? (str.right(min) != query.mid(j-min+1,min))
-                : (str.right(min).toLower() != query.mid(j-min+1,min))
-                )
+            int resStrLen = 0, resQueryLen = 0;
+            if ( !comparer( str.rightRef( min ), query.midRef( j - min + 1, min ),
+                            &resStrLen, &resQueryLen ) )
             {
                     // we not have matched
                     // this means we do not have a complete match
@@ -563,8 +616,8 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
 #endif
                     haveMatch=true;
                     ret->append( curEntity->transformedArea( matrix ) );
-                    j-=min;
-                    queryLeft-=min;
+                    j -= resStrLen;
+                    queryLeft -= resQueryLen;
                     if ( it_begin == TextList::ConstIterator() )
                     {
                         it_begin = it;
@@ -606,19 +659,35 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
 }
 
 QString TextPage::text(const RegularAreaRect *area) const
+{
+    return text(area, AnyPixelTextAreaInclusionBehaviour);
+}
+
+QString TextPage::text(const RegularAreaRect *area, TextAreaInclusionBehaviour b) const
 {
     if ( area && area->isNull() )
         return QString();
 
-    TextList::ConstIterator it = d->m_words.begin(), itEnd = d->m_words.end();
+    TextList::ConstIterator it = d->m_words.constBegin(), itEnd = d->m_words.constEnd();
     QString ret;
     if ( area )
     {
         for ( ; it != itEnd; ++it )
         {
-            if ( area->intersects( (*it)->area ) )
+            if (b == AnyPixelTextAreaInclusionBehaviour)
             {
-                ret += (*it)->text();
+                if ( area->intersects( (*it)->area ) )
+                {
+                    ret += (*it)->text();
+                }
+            }
+            else
+            {
+                NormalizedPoint center = (*it)->area.center();
+                if ( area->contains( center.x, center.y ) )
+                {
+                    ret += (*it)->text();
+                }
             }
         }
     }
