@@ -197,6 +197,9 @@ QTextDocument* Converter::convert( const QString &fileName )
         it.next();
 
         const QTextBlock block = mSectionMap[ it.key() ];
+        if ( !block.isValid() ) // local link without existing target
+          continue;
+
         Okular::DocumentViewport viewport = calculateViewport( mTextDocument, block );
 
         Okular::GotoAction *action = new Okular::GotoAction( QString(), viewport );
@@ -501,6 +504,9 @@ bool Converter::convertParagraph( const QDomElement &element )
             } else if ( childElement.tagName() == QLatin1String( "image" ) ) {
                 if ( !convertImage( childElement ) )
                     return false;
+            } else if ( childElement.tagName() == QLatin1String( "strikethrough" ) ) {
+                if ( !convertStrikethrough( childElement ) )
+                    return false;
             }
         } else if ( child.isText() ) {
             const QDomText childText = child.toText();
@@ -529,6 +535,22 @@ bool Converter::convertEmphasis( const QDomElement &element )
     return true;
 }
 
+bool Converter::convertStrikethrough( const QDomElement &element )
+{
+    QTextCharFormat origFormat = mCursor->charFormat();
+
+    QTextCharFormat strikeoutFormat( origFormat );
+    strikeoutFormat.setFontStrikeOut( true );
+    mCursor->setCharFormat( strikeoutFormat );
+
+    if ( !convertParagraph( element ) )
+        return false;
+
+    mCursor->setCharFormat( origFormat );
+
+    return true;
+}
+
 bool Converter::convertStrong( const QDomElement &element )
 {
     QTextCharFormat origFormat = mCursor->charFormat();
