@@ -238,10 +238,7 @@ void BookmarkList::slotBookmarksChanged( const KUrl& url )
         return;
 
     QTreeWidgetItem *item = itemForUrl( url );
-    if ( item )
-    {
-        selectiveUrlUpdate( url, item );
-    }
+    selectiveUrlUpdate( url, item );
 }
 
 QList<QTreeWidgetItem*> createItems( const KUrl& baseurl, const KBookmark::List& bmlist )
@@ -354,9 +351,11 @@ void BookmarkList::selectiveUrlUpdate( const KUrl& url, QTreeWidgetItem*& item )
     else
     {
         const QString fileString = url.isLocalFile() ? url.path() : url.prettyUrl();
+        bool fileitem_created = false;
+
         if ( item )
         {
-            for ( int i = item->childCount(); i >= 0; --i )
+            for ( int i = item->childCount() - 1; i >= 0; --i )
             {
                 item->removeChild( item->child( i ) );
             }
@@ -364,15 +363,27 @@ void BookmarkList::selectiveUrlUpdate( const KUrl& url, QTreeWidgetItem*& item )
         else
         {
             item = new QTreeWidgetItem( m_tree, FileItemType );
+            item->setText( 0, fileString );
+            item->setData( 0, UrlRole, qVariantFromValue( url ) );
+            fileitem_created = true;
+        }
+        if ( m_document->isOpened() && url == m_document->currentDocument() )
+        {
             item->setIcon( 0, KIcon( "bookmarks" ) );
             item->setExpanded( true );
-            item->setText( 0, fileString );
         }
         item->addChildren( createItems( url, urlbookmarks ) );
         if ( item != m_tree->invisibleRootItem() )
         {
             item->setToolTip( 0, i18ncp( "%1 is the file name", "%1\n\nOne bookmark", "%1\n\n%2 bookmarks", fileString, item->childCount() ) );
         }
+        if ( fileitem_created )
+        {
+            // we need to sort also the parent of the new file item,
+            // so it can be properly shown in the correct place
+            m_tree->invisibleRootItem()->sortChildren( 0, Qt::AscendingOrder );
+        }
+        item->sortChildren( 0, Qt::AscendingOrder );
     }
 
     connect( m_tree, SIGNAL( itemChanged( QTreeWidgetItem *, int ) ), this, SLOT( slotChanged( QTreeWidgetItem * ) ) );
